#ifndef SRCOMM3_Finout_h
#define SRCOMM3_Finout_h

#include <srcomm3/inout.h>

// This is all for handling errors.  It'd be great if everyone had
// C++ exceptions!
class FileSystem
{
  public:
	typedef void (*_ErrorHandler)( string s );
    friend class fileout;
    friend class filein;
    friend class recordfile;

  private:
    static _ErrorHandler ErrorHandler;
	static void HandleError( string msg, string arg );

  public:
    static void InstallErrorHandler( _ErrorHandler f = 0 );
    static _ErrorHandler GetErrorHandler();
};

class fileout: public standardout
{
  private:
    int handle, err, binary;
    int buf_len, buf_ptr;
    char *buf;

    void getbuffer();

  public:
	enum { OVERWRITE = 0, APPEND = 1 };

    fileout();
    fileout( string filename, int appendflag = 1, int binaryflag = 0, int wide = 0 );
    ~fileout();

    int open( string filename, int appendflag = 1, int binaryflag = 0 );
    int close();

    virtual void CC _put( int len, const char c[] );

    int flush();
    int exists() { return handle != -1; }
    int operator !();
};

class filein: public standardin
{
  private:
    int handle, err, binary;
    int buf_len, buf_ptr, buf_overflow;
    char *buf;

    void getbuffer();

  public:
    filein();
    filein( string filename, int binaryflag = 0 );
    ~filein();

    int open( string filename, int binaryflag = 0 );
    int close();
    int exists() { return handle != -1; }

    virtual CC char get();
    virtual CC char out( char c );
    virtual CC void begininput();
    virtual CC void endinput();
    virtual CC int keyhit();

    int operator !();
};

#endif
