#ifndef MTask_h
#define MTask_h

#ifndef __OS2__

// We're using setjmp and longjmp to implement coroutines, so overlays
// will NOT work
#include <setjmp.h>

// The Task object encapsulates a task; where the entire task system needs
// a global variable, we use static member variables instead.
class Task
{
	// The context is very conveniently stored within a jmp_buf
        jmp_buf jmp;

        static Task *current, *top, *killed;
        static int count;

	// Doubly linked round robin list
        Task* Next;
        Task* Prev;

	// Keep a pointer to the stack we've allocated
        char *stack;

	friend void _KillTask();
	friend void swap();

	long wakeup;

public:
	Task( void (*f)(), int stacksize = 2048 );
	virtual ~Task();

	virtual void swap();
        int exists()            { return Next != 0; }

        static void terminate();

        // wait() is similar to Borland's delay() function, except it allows
        // other tasks to run; to make existing programs work smoother, you
        // can #define delay wait
        friend void wait( long milliseconds );
};

// A ConTask is a Task that also remembers the colors, cursor location, and
// window setup when switching tasks.  Applications that need to store
// some sort of state can derive a new type of Task that stores & restores
// state on a task switch.
class ConTask: public Task
{
public:
	ConTask( void (*f)(), int stacksize = 2048 ):
		Task( f, stacksize ) {}
	~ConTask() {}

	// We have our own swap that saves & restores the screen state
	virtual void swap();
};

// swap() is a non-member function that switches tasks
void swap();

// mgetch() is similar to Borland's getch() function, except it lets
// other tasks run; you can #define getch mgetch if you'd like
int mgetch();

#endif

#endif	MTask_h
