#ifndef SRCOMM3_RecFile_h
#define SRCOMM3_RecFile_h

class recordfile
{
  protected:
    string pathname;
    int handle;
    int size;
	int err;

	// Optimization --- where was the last accessed file position?
	long lastaccess;

  public:
    recordfile( string filename, int sz );
    recordfile( int sz );
    ~recordfile();

    int open( string filename );
    int close();

    int CC write( long n, void *data, int count = 1 );
    int CC read( long n, void *data, int count = 1 );

    int CC eraseFile();

    int lock( long n );
    int unlock( long n );

    long CC filesize();
    const string& filename() const { return pathname; }
    int CC isopen() const { return handle != -1; }
    int CC exists() const { return handle != -1; }
    int CC error();
};

template<class T>
class RecordFile: private recordfile
{
  public:
    RecordFile( string filename )
        :recordfile( filename, sizeof(T) ) {}
    RecordFile()
        :recordfile( sizeof(T) ) {}
    ~RecordFile() {}

	void open( string filename ) { recordfile::open( filename ); }
	void close()                 { recordfile::close(); }

    int scan( T& data, long p )
        { return read( p, &data, 1 ) > 0; }
    int load( T& data, long p )
        { lock( p ); return read( p, &data, 1 ) > 0; }
    int save( T& data, long p )
		{ if( write( p, &data, 1 ) > 0 ) { unlock( p ); return 1; } return 0; }
    int abort( long p )
        { unlock( p ); return 1; }

    long CC filesize()          { return recordfile::filesize(); }
    const string& filename() const { return recordfile::filename(); }
    int CC isopen() const       { return recordfile::isopen(); }
    int CC exists() const       { return recordfile::exists(); }
    int CC error()              { return recordfile::error(); }
};

#endif
