//////////////////////////////////////////////////////////////////////
// SCRANSI                                                          //
//////////////////////////////////////////////////////////////////////
// ScreenAnsiOutput interfaces to AnsiOutput to turn ANSI sequences //
// into commands to the console (screen).                           //
//////////////////////////////////////////////////////////////////////

#include <dos.h>
#include <bios.h>

#include <cstring.h>
#include <unit/StrUtl.h>

#include <srcomm3/Ansi.h>

#include <srcomm3/coninout.h>
#include <srcomm3/desqview.h>

///////////////////////////////ScreenAnsiOutput///////////////////////////

class ScreenAnsiOutput: public AnsiOutput
{
  private:
    int height_, width_;

    void gotoxy( int x, int y );
    void clreol();
    void clrscr( int y1 = -1, int y2 = -1 );
    void setattr( int a );
    int whereX();
    int whereY();
    void beep();
    void putchar( char c );
    int width();
    int height();
    int currentcolor();

    friend void ansi_init();

  public:
    ScreenAnsiOutput();
    ~ScreenAnsiOutput();
};

// Conio-specific
void ScreenAnsiOutput::setattr( int a ) { sout.SetAttr( a ); }
void ScreenAnsiOutput::clreol() { sout.ClearEOL(); }
void ScreenAnsiOutput::gotoxy( int x, int y ) { sout.GotoXY( x, y ); }
int ScreenAnsiOutput::whereX() { return sout.WhereX(); }
int ScreenAnsiOutput::whereY() { return sout.WhereY(); }
int ScreenAnsiOutput::width() { return width_; }
int ScreenAnsiOutput::height() { return height_; }
int ScreenAnsiOutput::currentcolor() { return sout.GetAttr(); }
void ScreenAnsiOutput::putchar( char c ) { sout << c; }

void ScreenAnsiOutput::beep()
{
	sound( 2000 );
	TimedDelay( 75 );
	nosound();
}

void ScreenAnsiOutput::clrscr( int Y1, int Y2 )
{
	if( Y1 == -1 )
	{
		sout.Clear();
		return;
	}

    int x1, y1, x2, y2;
    byte x, y;

    sout.GetRegion( x1, y1, x2, y2 );
    x = sout.WhereX();
    y = sout.WhereY();
    sout.Region( x1, Y1+y1-1, x2, Y2+y1-1 );
    sout.Clear();
    sout.Region( x1, y1, x2, y2 );
    sout.GotoXY( x, y );
}

ScreenAnsiOutput::ScreenAnsiOutput()
{
//    if( DetectDESQview() || DetectMWindows() )
//        conout::BIOSVideo();
}

ScreenAnsiOutput::~ScreenAnsiOutput()
{
    sout.Cursor();
    sout.NoRegion();
    sout.GotoXY( 1, height_ );
    sout.ClearEOL();
    sout.GotoXY( 1, height_-1 );
}

static ScreenAnsiOutput _aout;
AnsiOutput* aout = &_aout;

void ansi_init()
{
    extern byte screen_cols;
    extern byte screen_rows;

    _aout.width_ = screen_cols;
    _aout.height_ = screen_rows;
    if( _aout.height_ < 12 )     _aout.height_ = 25;
    if( _aout.width_ < 80 )    _aout.width_ = 80;

    sout.Clear();
    sout.Region( 1, 2, _aout.width_, _aout.height_-1 );
    sout.Cursor( 0, 255 );
}

