//////////////////////////////////////////////////////////////////////
// SRINOUT                                                          //
//////////////////////////////////////////////////////////////////////
// Oh oh... what a mess this unit is!                               //
//                                                                  //
// This lets you send output to both the modem and the screen.  The //
// screen output gets sent to ScreenAnsiOutput (SCRANSI.CPP), while //
// modem output, if the user is remote, is sent to the Fossil class //
// (FOSSIL.CPP).  There are variables for controlling ANSI, IBM     //
// high ascii characters, special effects, and pausing.  One goodie //
// is the KeyIntercept class; it allows you to intercept local      //
// keystrokes to call your functions instead of passing them on as  //
// normal input to the program.                                     //
//////////////////////////////////////////////////////////////////////

#pragma hdrstop

#include <dos.h>
#include <signal.h>

#include <bios.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>

#include <cstring.h>

#include <srcomm3/fossil.h>
#include <srcomm3/ansi.h>
#include <srcomm3/desqview.h>
#include <srcomm3/inout.h>
#include <srcomm3/finout.h>
#include <srcomm3/srinout.h>
#include <srcomm3/srparam.h>

#define TimedDelay(n) delay(n)

GameInput::Hook GameInput::multitask_hook = 0;     // Multitasking code hook function
int GameInput::NoKeyTimeOut = 300;      // Time (seconds) before keyboard
										// timeout occurs

// Set this to non-NUL to fake a keypress
char GameInput::StuffedChar = '\0';
// Last key read from Modem or Keyboard?
GameInput::InputDevice GameInput::LastRead = Keyboard;

int GameOutput::Ansi = 0;               // Set to 1 if ansi screen
int GameOutput::ComPort = 0;            // 0 = local
                                        // 1..8 = com port #
int GameOutput::IBMCharacters = 1;      // IBM character set available

int GameOutput::SpecialEffects = 1;     // Pause whirly cursor
int GameOutput::LinesPrinted = 0;       // Number of lines printed
int GameOutput::PageLength = 21;        // Page length

// Colors
attr aNormal( 7 );
attr aCursor( 15 );

// Standard keyboard-in/out object instances
GameInput kin;
GameOutput kout;
LocalOutput local;
RemoteOutput remote;

const char* GameOutput::LogFilePrefix = "";

struct _PauseColors PauseColor = { 0, 7, 15 };

fileout LogFile;

static int keybhit()
{
	return bioskey( 1 ) != 0;
}

static int getkey()
{
	int x = bioskey( 0 );
	if( ( x & 0xff ) != 0 )
		x = x & 0xff;
	return x;
}

////////////////////////////////////////////////////////////GameKeyIntercept

// GameKeyIntercept is a class that intercepts keystrokes and runs a Hook
// procedure with it.  For example, the Chat procedure creates a new
// GameKeyIntercept object as F10, but if that module is not linked in,
// the chat won't be installed.
//
// Example use:
//      (A)  To capture a key in a particular procedure (temporary)
//           void foo()
//           {
//               GameKeyIntercept F10( MyChatProcedure, 0, 68 );
//               GameKeyIntercept CtrlA( MyCtrlAProcedure, 1 );
//           }
//
//      (B)  To capture a key globally (permanent)
//               ...
//               GameKeyIntercept* F10 =
//                        new GameKeyIntercept( ChatProcedure, 0, 68 );
//               ...
//           If the procedure can occur ANY time (having no access of kin
//           and kout), you can add this:
//               F10 -> Safe();
//           and SR-COMM will run your procedure at more times.
//

GameKeyIntercept* GameKeyIntercept::head = 0;

GameKeyIntercept::GameKeyIntercept( GameInput::Hook f, unsigned k, unsigned scan )
{
    anytime = 0;
    hook = f;
    next = head;
    head = this;
	key = scan * 256u + k;
}

GameKeyIntercept::~GameKeyIntercept()
{
    if( head == this )
	{
        head = next;
        return;
    }

    GameKeyIntercept* p = head;
    while( p && p -> next )
	{
        if( p -> next == this )
        {
            p -> next = next;
            return;
        }
		p = p -> next;
    }
}

GameInput::Hook GameKeyIntercept::Search( unsigned k, int anytime )
{
    GameKeyIntercept* p = head;
	while( p )
    {
        if( p -> key == k )
            if( p -> anytime || anytime )
                return p -> hook;
        p = p -> next;
	}
    return 0;
}

GameInput::Hook GameKeyIntercept::SearchExtended( unsigned scan, int anytime )
{
    return Search( scan * 256u, anytime );
}

//////////////////////////////////////////////////////////////GameKeyIntercept


/////////////////////////////////////////////////////////////////////GameInput

void stop_key()
{
    raise( SIGTERM );
}

void break_key()
{
    raise( SIGINT );
}

GameInput::GameInput(): standardin()
{
	frozen = 0;
}

GameInput::~GameInput()
{
}

char CC GameInput::out( char c )
{
	kout << c;
	return c;
}

void CC GameInput::begininput()
{
    savefore = kout.GetForeground();
	saveback = kout.GetBackground();
	kout << aCursor;
    if( aCursor.back > 0 && GameOutput::Ansi )
    {
		int rawmode = kout.RawMode();
        int useansi = kout.UsingAnsi();
		kout.SetRawMode();
        kout.SetAnsi( 1 );
		if( width() > 0 )
        {
            for( int i = 0; i < width(); i++ )
                kout << ' ';
			if( GameOutput::Ansi )
                kout << "\033[" << width() << 'D';
            else
                for( i = 0; i < width(); i++ )
					kout << '\b';
        }
        if( !rawmode ) kout.ResetRawMode();
        kout.SetAnsi( useansi );
	}
}

void CC GameInput::endinput()
{
	kout << attr( savefore, saveback );
}

int multitask()
{
	// let DesqView or MS-Windows have more time
//	TimeSlice();

	if( GameInput::multitask_hook )
	{
		(*GameInput::multitask_hook)();
        return 1;
    }
    return 0;
}

int CC GameInput::keyhit()
{
	for(;;)
    {
        int predict = bioskey( 1 );
		if( predict == -1 )
			break_key();

        {
            static int howmany = 0;
            if( howmany++ == 10 )
            {
                multitask();
                howmany = 0;
            }
		}

        if( predict )
		{
			Hook h = GameKeyIntercept::Search( predict );
			if( h )
            {
                bioskey( 0 );
				(*h)();
			}
			predict = 0;
		}

		if( GameOutput::ComPort != 0 && !Fossil::carrierdetect() )
		{
			local << attr(14) << "Hangup?\n";
			exit(1);
		}

		if( !predict )
            return( !frozen && ( Fossil::buffer() || ( keybhit() && kout.LocalOutput ) ) );
    }
}

//static Resource UserBreak( "UserBreak", 1L );

char CC GameInput::get()
{
	static int extendflag = 0;
	char promptchar = GameOutput::IBMCharacters? '' : '_';
	char* backspace = GameOutput::Ansi? "\033[D" : "\b";
	int rawmode = kout.RawMode();
    int useansi = kout.UsingAnsi();
	int allowBreak = 0; /* UserBreak.Boolean() */

	long t = 0;
	int ch;

    if( frozen )  return 0;

	GameOutput::LinesPrinted = 0;

	if( extendflag )
	{
		int t = extendflag;
		extendflag = 0;
		return t;
	}

    kout.SetAnsi( 1 );
	if( !rawmode )
	{
		kout.SetRawMode();
		kout << promptchar << backspace;
		kout.ResetRawMode();
	}

	time_t start_time, end_time;
	int warned = 0;

	time( &start_time );
	time( &end_time );

	for( ; start_time + 30 + NoKeyTimeOut > end_time; t++ )
	{
		if( GameOutput::ComPort != 0 && !Fossil::carrierdetect() )
			start_time--;

		if( keyhit() || StuffedChar != '\0' )
		{
			if( StuffedChar != '\0' )
			{
				LastRead = Keyboard;
				if( !rawmode ) kout << " \b";
				char c = StuffedChar;
				StuffedChar = '\0';
                kout.SetAnsi( useansi );
				return c;
			}

			if( GameOutput::ComPort != 0 && Fossil::buffer() )
			{
				LastRead = Modem;

				ch = Fossil::read();
				if( !rawmode && ch == 127 ) ch = '\b';
				if( !rawmode ) kout << " \b";
				if( ch == 3 && allowBreak )
				{
					LineBuffer oldline = kout.Buffer();
					int oldfore = kout.GetForeground();
					int oldback = kout.GetBackground();

					kout << attr() << "\nBreak out of Program?  [y/N]";
					for(;;)
					{
						char c;
						kin >> c;
						if( c == 'Y' || c == 'y' )
							break_key();
						if( c == 'N' || c == 'n' || c == '\r' )
							break;
					}

					if( !rawmode ) kout << attr() << oldline << attr(oldfore,oldback);
				}
				else if( ch == 4 && allowBreak )
				{
					LineBuffer oldline = kout.Buffer();
					int oldfore = kout.GetForeground();
					int oldback = kout.GetBackground();

					kout << attr() << "\nStop Program?  [y/N]";
					for(;;)
					{
						char c;
						kin >> c;
						if( c == 'Y' || c == 'y' )
							stop_key();
						if( c == 'N' || c == 'n' || c == '\r' )
							break;
					}

					if( !rawmode ) kout << attr() << oldline << attr(oldfore,oldback);
				}
                else
                {
                    kout.SetAnsi( useansi );
                    return ch;
                }
				if( !rawmode )
				{
					kout.SetRawMode();
					kout << promptchar << backspace;
					kout.ResetRawMode();
				}
			}

			if( keybhit() )
			{
				LastRead = Keyboard;

				ch = getkey();

				if( ( ch & 0xff ) == 0 )
				{
					ch >>= 8;

                    Hook h = GameKeyIntercept::SearchExtended( ch );
					if( h )
					{
						(*h)();
						continue;
					}

					if( !rawmode ) kout << " \b";

                    h = GameKeyIntercept::SearchExtended( ch, 1 );
                    if( h )
						(*h)();
                    else
                    {
                        extendflag = ch;
                        kout.SetAnsi( useansi );
                        return 0;
					}
                }
				else
                {
                    Hook h = GameKeyIntercept::Search( ch );
                    if( h )
                    {
                        (*h)();
						continue;
                    }

                    if( !rawmode ) kout << " \b";

					h = GameKeyIntercept::Search( ch, 1 );
                    if( h )
						(*h)();
                    else if( ch == 3 )
                        break_key();
                    else if( ch == 4 )
                        stop_key();
                    else if( kout.LocalOutput )
                    {
                        kout.SetAnsi( useansi );
                        return ch;
                    }
                }
                if( !rawmode )
                {
					kout.SetRawMode();
					kout << promptchar << backspace;
                    kout.ResetRawMode();
                }
            }
        }

		time_t new_time;
		time( &new_time );
		if( new_time - end_time > 5 )
			// if 5 seconds have elasped, it *might* be paused
			// and we want to respect their wishes
			start_time += ( new_time - end_time );
		end_time = new_time;

		if( end_time - start_time > NoKeyTimeOut && !warned )
		{
			LineBuffer oldline = kout.Buffer();
            int oldfore = kout.GetForeground();
            int oldback = kout.GetBackground();

            if( !rawmode )
			{
				kout << attr(14) << " \n\aWarning!  Program will abort in 30 seconds if you do not respond...\n";
                kout << attr() << oldline << attr(oldfore,oldback);
			}
			else
				kout << "\a\a\a";
			warned = 1;
		}

		// Call the multi-tasking hook function if one exists
		multitask();
        TimeSlice();
		TimedDelay( 100 );
	}

    if( GameOutput::ComPort != 0 )
        Fossil::deinit();
	t = GameOutput::ComPort;
	GameOutput::ComPort = 0;
	kout << attr(14) << " \a\nTIMEOUT:  COM" << t << ": no input for " << attr(15) << (long)(end_time - start_time) << attr(14) << " seconds\n";
	exit( 2 );
	return 0;
}

/////////////////////////////////////////////////////////////////////GameOutput

GameOutput::GameOutput(): standardout( 79 )
{
	LogFileOpen = 0;
	LogFileOutput = 0;
	LocalOutput = 1;
	RemoteOutput = 1;
    UseAnsi = 0;
    setColorTranslator( 0 );
}

GameOutput::~GameOutput()
{
    if( LogFileOpen )     CloseLogFile();
}

void GameOutput::OpenLogFile( string filename )
{
	if( LogFileOpen )
		LogFile.close();
	LogFileOpen = 1;
	LogFile.open( filename, 0 );
    if( Ansi ) LogFile.setColorTranslator( AnsiTranslator );
	LogFileOutput = 1;
}

void GameOutput::CloseLogFile()
{
	LogFile.close();
	LogFileOpen = 0;
    LogFileOutput = 0;
}

void CC GameOutput::_put( int len, const char c[] )
{
    bool PauseFlag = false;

    if( !rawmode )
    {
        for( int i = 0; i < len; i++ )
            if( c[i] == '\n' )
                LinesPrinted++;
        if( LinesPrinted > PageLength && PageLength > 12 )
        {
			LinesPrinted = 0;
            PauseFlag = true;
		}
    }

#if 0
    if( LogFileOutput && c == '\n' )
        LogFile << attr(8) << LogFilePrefix << Buffer() << '\n';
#endif

    if( LocalOutput )
        local.put( len, c );
	if( RemoteOutput && ComPort != 0 )
        remote.put( len, c );
    if( PauseFlag && !kin.Frozen() )
		pause();
}

#include <srcomm3/coninout.h>

void CC GameOutput::SetAttr( int f, int b )
{
    new_fore = current_fore = f;
    if( b != -1 )
        new_back = current_back = b;
    else
        new_back = current_back;
    if( UseAnsi )
        local.SetAttr( f, b );
    else
        sout.SetAttr( f, b );
    remote.SetAttr( f, b );
}

/////////////////////////////////////////////////////////////////////Local

LocalOutput::LocalOutput(): standardout()
{
}

LocalOutput::~LocalOutput()
{
}

void GameOutput::SetAnsi( int flag )
{
	UseAnsi = flag;
}

void CC LocalOutput::_put( int len, const char c[] )
{
	if( kout.UsingAnsi() )
	{
		for( int i = 0; i < len; i++ )
			aout -> put( c[i] );
	}
	else
		sout._put( len, c );
}

/////////////////////////////////////////////////////////////////////Remote
RemoteOutput::RemoteOutput(): standardout()
{
}

RemoteOutput::~RemoteOutput()
{
}

void CC RemoteOutput::_put( int len, const char s[] )
{
	if( GameOutput::ComPort == 0 )    return;

    for( int i = 0; i < len; i++ )
    {
        char c = s[i];

        if( !GameOutput::IBMCharacters )
        {
            if( c == '' || c == '' )
                c = '|';
            else if( c == '' )
                c = '-';
            else if( c == '' )
                c = '=';
            if( (unsigned char)c > 127 )
                c = '#';
        }

        if( !Fossil::carrierdetect() || Fossil::write( c ) )
        {
            int t = GameOutput::ComPort;

            Fossil::hangup();
            Fossil::deinit();

            GameOutput::ComPort = 0;
            kout << "\a\nHANGUP:  COM" << t << ": could not write character\n";
            exit( 2 );
			return;
        }
    }
}

// Pause for a while (until a key is pressed)

void pause()
{
	char c;
	int t = 0;

	if( kin.Frozen() ) return;

	int SaveLogFileOutput = kout.GetLogFileOutput();
	kout.PutLogFileOutput( 0 );

	int SaveRawMode = kout.RawMode();
	kout.SetRawMode();

	if( !kin.keyhit() )
	{
		int f = kout.GetForeground(), b = kout.GetBackground();

		if( GameOutput::IBMCharacters )
		{
			kout << attr(PauseColor.bg) << "" << attr(PauseColor.bc,PauseColor.bg) << "PAUSED";
			kout << attr(PauseColor.bg,0) << "۲" << attr(13) << "  ";
		}
		else
			kout << attr(PauseColor.bg) << "" << attr(PauseColor.fg) << ""
                 << attr(PauseColor.bc) << "Paused" << attr(PauseColor.fg) << ""
                 << attr(PauseColor.bg) << "" << attr(13) << "  ";

		while( GameOutput::SpecialEffects && !kin.keyhit() && t < 100 )
		{
            if( GameOutput::IBMCharacters )
			{
				kout << attr( 11 ) << "\b/";
                multitask(); TimedDelay( 50 );
				kout << attr( 1 ) << "\bX";
				multitask(); TimedDelay( 50 );
				kout << attr( 14 ) << "\b\\";
                multitask(); TimedDelay( 50 );
				kout << attr( 4 ) << "\bX";
                multitask(); TimedDelay( 50 );
			}
			else
			{
				kout << attr( 14 ) << "\b.";
                multitask(); TimedDelay( 36 );
				kout << attr( 12 ) << "\bo";
                multitask(); TimedDelay( 50 );
				kout << attr( 4 )  << "\bO";
                multitask(); TimedDelay( 50 );
			}
			t++;
		}

		kout << "\b ";
		if( !kin.keyhit() ) kout << attr(15);
		kin >> c;
		kout << attr( f, b );
		for( t = 0; t < 16; t++ )
			kout << "\b \b";
	}
	else kin >> c;

	GameOutput::LinesPrinted = 0;

	if( !SaveRawMode ) kout.ResetRawMode();
	kout.PutLogFileOutput( SaveLogFileOutput );
}

void cdecl signal_INT( int code )
{
	GameOutput::PageLength = 0;
	kout << '\n' << attr(12) << '[' << attr(14) << "INTERRUPT" << attr(12) << ']' << attr() << '\n';
	if( code == 999 ) return;
	exit( 1 );
}

void cdecl signal_TERM( int code )
{
	GameOutput::PageLength = 0;
	local << '\n' << attr(12) << '[' << attr(14) << "GAME TERMINATED" << attr(12) << ']' << attr() << '\n';
    if( code == 999 ) return;
    exit( 0 );
}

void init_trapping()
{
	signal( SIGINT, signal_INT );
    signal( SIGTERM, signal_TERM );
}

#pragma startup init_trapping 99

GameInput::Hook GameInput::InstallHook( GameInput::Hook new_hook )
{
	GameInput::Hook old_hook = multitask_hook;

	multitask_hook = new_hook;
	return old_hook;
}

