#ifndef SRCOMM3_Srinout_h
#define SRCOMM3_Srinout_h

#include <srcomm3/inout.h>

// Normal output and normal input colors
extern attr aNormal, aCursor;

// Pause until a key is pressed
void pause();

// Pause Color Customization
struct _PauseColors
{
    unsigned char bg, fg, bc;     // background, foreground, bright color
};
extern _PauseColors PauseColor;

class GameInput: public standardin
{
  private:
	int savefore, saveback;
    int frozen;
  public:
    enum InputDevice { Keyboard, Modem };
	static InputDevice LastRead;

    // multitask_hook is called whenever we are waiting on the user to press keys
	typedef void (*Hook)();
	static Hook multitask_hook;
	static Hook InstallHook( Hook new_hook );

	static int NoKeyTimeOut;
	static char StuffedChar;

	GameInput();
	~GameInput();

	virtual void CC begininput();
	virtual void CC endinput();

	virtual int CC keyhit();
	virtual char CC get();
	virtual char CC out( char c );

    void Freeze()       { frozen = 1; }
    void Melt()       { frozen = 0; }
    int  Frozen()       { return frozen; }
};

class GameOutput: public standardout
{
  private:
	int LogFileOpen;
	int LogFileOutput;
    int UseAnsi;

	friend class keyin;
  public:
	static int Ansi;
	static int ComPort;
	static int IBMCharacters;
	static int PageLength;
	static int SpecialEffects;
	static int LinesPrinted;

	static const char* LogFilePrefix;
	int LocalOutput;
    int RemoteOutput;

	GameOutput();
	~GameOutput();

    virtual void CC _put( int len, const char c[] );
    virtual void CC SetAttr( int f, int b = -1 );

    void SetAnsi( int flag );
    int  UsingAnsi() { return UseAnsi; }

	void OpenLogFile( string filename );
    void CloseLogFile();

    void EnableLogFileOutput()    { if( LogFileOpen ) LogFileOutput = 1; }
    void DisableLogFileOutput()   { LogFileOutput = 0; }
    int  GetLogFileOutput()     { return LogFileOutput; }
    void PutLogFileOutput( int x )  { if( LogFileOpen ) LogFileOutput = x; }
};

class LocalOutput : public standardout
{
  public:
	LocalOutput();
	~LocalOutput();

    virtual void CC _put( int len, const char c[] );
};

class RemoteOutput : public standardout
{
  public:
	RemoteOutput();
	~RemoteOutput();

    virtual void CC _put( int len, const char c[] );
};

extern GameInput kin;
extern GameOutput kout;
extern LocalOutput local;
extern RemoteOutput remote;

class GameKeyIntercept
{
  private:
    static GameKeyIntercept* head;
           GameKeyIntercept* next;

    unsigned key;
    int anytime;
    GameInput::Hook hook;
  public:
	GameKeyIntercept( GameInput::Hook f, unsigned k, unsigned scan = 0 );
	~GameKeyIntercept();

    void Safe()       { anytime = 1; }
    void UnSafe()       { anytime = 0; }

    // These take a key and search for the procedure to run; anytime = 0
    // will run any Safe key; anytime = 1 will run any key
    static GameInput::Hook Search( unsigned k, int anytime = 0 );
    static GameInput::Hook SearchExtended( unsigned scancode, int anytime = 0 );
};

class AnsiOutput;
extern AnsiOutput* aout;

#endif

