// VGA unit (text mode)
//
// Palette functions adapted from Turbo Pascal - PC Techniques, June/July 1990

#pragma hdrstop
#include <conio.h>
#include <dos.h>
#include "vga.h"

static unsigned char vga_EGA2VGA[16] = {0,1,2,3,4,5,20,7,56,57,58,59,60,61,62,63};
enum Adapter { unknown, Mono, CGA, EGAMono, EGACol, VGAMono, VGACol, MCGAMono, MCGACol };
static Adapter videocard;
static int changed = 0;

static struct { unsigned char r, g, b; } far RGB[64];

static void GetAllRGB()
{
	struct SREGS sr;
	union REGS r;
	sr.es = FP_SEG( &RGB );
	r.x.dx = FP_OFF( &RGB );
	r.x.cx = 64;
	r.x.bx = 0;
	r.x.ax = 0x1017;
	int86x( 0x10, &r, &r, &sr );
}

static void SetAllRGB()
{
	if( !HasVGA() ) return;
	if( !changed ) return;

	struct SREGS sr;
	union REGS r;

	// Reset video mode
	textmode( -1 );

	sr.es = FP_SEG( &RGB );
	r.x.dx = FP_OFF( &RGB );
	r.x.cx = 64;
	r.x.bx = 0;
	r.x.ax = 0x1012;
	int86x( 0x10, &r, &r, &sr );
}

void SetRGBPalette()
{
	for( int i = 0; i < 16; i++ )
		SetRGBPalette( i );
}

void SetRGBPalette( int i )
{
	if( i < 0 || i > 15 ) return;

	int c = vga_EGA2VGA[i];
	int r = RGB[c].r;
	int g = RGB[c].g;
	int b = RGB[c].b;
	SetRGBPalette( i, 4 * r, 4 * g, 4 * b );
}

void SetRGBPalette( int i, int r, int g, int b )
{
	if( !HasVGA() || i < 0 || i > 15 ) return;
		changed = 1;
	if( r < 0 ) r = 0; if( r > 0xff ) r = 0xff;
	if( g < 0 ) g = 0; if( g > 0xff ) g = 0xff;
	if( b < 0 ) b = 0; if( b > 0xff ) b = 0xff;

	r /= 4; g /= 4; b /= 4;
	i = vga_EGA2VGA[i];

	_DH = r & 0xff;
	_CH = g & 0xff;
	_CL = b & 0xff;
	_BX = i;
	_AX = 0x1010;
	geninterrupt( 0x10 );
}

void SetVGABorder( int col )
{
	if( !HasVGA() ) return;
	_AX = 0x1001;
	_BH = col;
	geninterrupt( 0x10 );
}

int HasVGA()
{
	if( videocard != VGAMono && videocard != VGACol
	 && videocard != MCGAMono && videocard != MCGACol )
		return 0;
	return 1;
}

int LoadVGAFont( char far *buf, int ascii, int num )
{
	if( videocard != VGAMono && videocard != VGACol
	 && videocard != MCGAMono && videocard != MCGACol )
		return 0;

	_AX = 0x1100;
	_BX = 0x1000;
	_CX = num;
	_DX = ascii;
	_ES = (unsigned)(FP_SEG(buf));
	_BP = (unsigned)(FP_OFF(buf));
	geninterrupt( 0x10 );
	return 0;
}

void DetectCard()
{
	union REGS r;
	r.h.al = 0;
	r.h.ah = 0x1A;
	int86( 0x10, &r, &r );

	videocard = unknown;
	if( r.h.al == 0x1A )
	{
		switch( r.h.bl )
		{
			case 0:	videocard = unknown;    break;
			case 1: videocard = Mono;       break;
			case 4: videocard = EGACol;     break;
			case 5: videocard = EGAMono;    break;
			case 7: videocard = VGAMono;    break;
			case 8: videocard = VGACol;     break;
			case 10:
			case 12:videocard = MCGACol;    break;
			case 11:videocard = MCGAMono;   break;
			default:videocard = CGA;
		}
	}
	else
	{
		r.h.ah = 0x12;
		r.x.bx = 0x10;
		int86( 0x10, &r, &r );
		if( r.x.bx == 0x10 )
		{
			int86( 0x11, &r, &r );
			if( ( ( r.h.al & 0x30 ) >> 4 ) == 3 )
				videocard = Mono;
			else
				videocard = CGA;
		}
		else
		{
			r.h.ah = 0x12;
			r.x.bx = 0x10;
			int86( 0x10, &r, &r );
			if( r.h.bh == 0 )
				videocard = EGACol;
			else
				videocard = EGAMono;
		}
	}

		if( HasVGA() ) GetAllRGB();
}

#pragma startup DetectCard 99
#pragma exit SetAllRGB
